#ifndef FLS_STD_UNORDERED_MAP_HPP
#define FLS_STD_UNORDERED_MAP_HPP

#include <unordered_map>

namespace fastlanes {
template <typename T, typename Y>
using unordered_map = std::unordered_map<T, Y>;
}
#endif // FLS_STD_UNORDERED_MAP_HPP
