#ifndef FLS_COR_EXP_VARIANT_HPP
#define FLS_COR_EXP_VARIANT_HPP

#include <cstdint>
namespace fastlanes {
// enum class var_t : uint8_t { // NOLINT.
//	UNDECIDED = 0, // reserved
// };
} // namespace fastlanes
#endif // FLS_COR_EXP_VARIANT_HPP
