#ifndef FLS_STD_UNORDERED_SET_HPP
#define FLS_STD_UNORDERED_SET_HPP

#include <unordered_set>

namespace fastlanes {
template <typename T>
using unordered_set = std::unordered_set<T>;
}
#endif // FLS_STD_UNORDERED_SET_HPP
