#ifndef FLS_API_EXP_HPP
#define FLS_API_EXP_HPP

#include "fls/cor/exp/alp/alp_exp.hpp"
#include "fls/cor/exp/byte_arr/byte_arr_exp.hpp"
#include "fls/cor/exp/delta/delta_exp.hpp"
#include "fls/cor/exp/dict/dict_exp.hpp"
#include "fls/cor/exp/dict_rle/dict_rle_exp.hpp"
#include "fls/cor/exp/ffor/ffor_exp.hpp"
#include "fls/cor/exp/fsst/fsst_exp.hpp"
#include "fls/cor/exp/fsst12/fsst12_exp.hpp"
#include "fls/cor/exp/rle/rle_exp.hpp"
#include "fls/cor/exp/uncompressed/uncompressed_exp.hpp"

#endif // FLS_API_EXP_HPP
