#ifndef FLS_STD_STRING_HPP
#define FLS_STD_STRING_HPP

#include <string>
#include <string_view>

namespace fastlanes {
using string      = std::string;
using string_view = std::string_view;
} // namespace fastlanes

#endif // FLS_STD_STRING_HPP
