#ifndef FLS_API_PRM_HPP
#define FLS_API_PRM_HPP

#include "fls/cor/prm/alp_prm.hpp"
#include "fls/cor/prm/b_ffor_prm.hpp"
#include "fls/cor/prm/bitpack.hpp"
#include "fls/cor/prm/factor.hpp"
#include "fls/cor/prm/ffor_prm.hpp"
#include "fls/cor/prm/fsst/fsst_prm.hpp"
#include "fls/cor/prm/mem_cpy_prm.hpp"
#include "fls/cor/prm/mem_trs.hpp"
#include "fls/cor/prm/patch/b_patch.hpp"
#include "fls/cor/prm/patch/ll_patch.hpp"
#include "fls/cor/prm/patch/s_patch.hpp"
#include "fls/cor/prm/prm_type.hpp"
#include "fls/cor/prm/rle_prm.hpp"
#include "fls/cor/prm/rsum_prm.hpp"
#include "fls/cor/prm/trs_prm.hpp"

#endif // FLS_API_PRM_HPP
