#include "fls/cor/prm/factor.hpp" // for pack
#include <cstdint>                // for uint64_t, uint32_t, uint16_t, uint8_t
#include <sys/cdefs.h>            // for __restrict

namespace fastlanes { namespace factor {
}}