#ifndef FLS_IO_MMAP_HPP
#define FLS_IO_MMAP_HPP

#include "fls/common/common.hpp"
#include <filesystem>

namespace fastlanes {
class Mmap {
public:
};
} // namespace fastlanes

#endif // FLS_IO_MMAP_HPP
