#ifndef FLS_PRIMITIVE_BITPACK_BITPACK_HPP
#define FLS_PRIMITIVE_BITPACK_BITPACK_HPP

#include "fls/common/common.hpp"

namespace fastlanes {
n_t calculate_bitpacked_vector_size(bw_t bw);
}
#endif // FLS_PRIMITIVE_BITPACK_BITPACK_HPP
