// automatically generated by the FlatBuffers compiler, do not modify

#ifndef FLATBUFFERS_GENERATED_OPERATORTOKEN_FASTLANES_H_
#define FLATBUFFERS_GENERATED_OPERATORTOKEN_FASTLANES_H_

#include "flatbuffers/flatbuffers.h"

// Ensure the included flatbuffers.h is the same version as when this file was
// generated, otherwise it may not be compatible.
static_assert(FLATBUFFERS_VERSION_MAJOR == 25 && FLATBUFFERS_VERSION_MINOR == 2 && FLATBUFFERS_VERSION_REVISION == 10,
              "Non-compatible flatbuffers version included");

namespace fastlanes {

enum class OperatorToken : uint16_t {
	INVALID                              = 0,
	EXP_EQUAL                            = 1,
	EXP_CONSTANT_I64                     = 2,
	EXP_CONSTANT_I32                     = 3,
	EXP_CONSTANT_I16                     = 4,
	EXP_CONSTANT_DBL                     = 5,
	EXP_CONSTANT_STR                     = 6,
	EXP_CONSTANT_U08                     = 7,
	EXP_STRUCT                           = 8,
	EXP_UNCOMPRESSED_STR                 = 9,
	EXP_UNCOMPRESSED_I64                 = 10,
	EXP_UNCOMPRESSED_I16                 = 11,
	EXP_UNCOMPRESSED_I32                 = 12,
	EXP_UNCOMPRESSED_DBL                 = 13,
	EXP_UNCOMPRESSED_U08                 = 14,
	EXP_FFOR_I64                         = 15,
	EXP_FFOR_I32                         = 16,
	EXP_FFOR_I16                         = 17,
	EXP_DICT_I64_FFOR_U32                = 18,
	EXP_DICT_I64_FFOR_U16                = 19,
	EXP_DICT_I64_FFOR_U08                = 20,
	EXP_DICT_I32_FFOR_U32                = 21,
	EXP_DICT_I32_FFOR_U16                = 22,
	EXP_DICT_I32_FFOR_U08                = 23,
	EXP_DICT_I16_FFOR_U16                = 24,
	EXP_DICT_I16_FFOR_U08                = 25,
	EXP_DICT_DBL_FFOR_U32                = 26,
	EXP_DICT_DBL_FFOR_U16                = 27,
	EXP_DICT_DBL_FFOR_U08                = 28,
	EXP_DICT_STR_FFOR_U32                = 29,
	EXP_DICT_STR_FFOR_U16                = 30,
	EXP_DICT_STR_FFOR_U08                = 31,
	EXP_FSST_DICT_STR_FFOR_U32           = 32,
	EXP_FSST_DICT_STR_FFOR_U16           = 33,
	EXP_FSST_DICT_STR_FFOR_U08           = 34,
	WIZARD_DICTIONARY_ENCODE             = 35,
	EXP_DICT_I64_U32                     = 36,
	EXP_DICT_I64_U16                     = 37,
	EXP_DICT_I64_U08                     = 38,
	EXP_DICT_I32_U32                     = 39,
	EXP_DICT_I32_U16                     = 40,
	EXP_DICT_I32_U08                     = 41,
	EXP_DICT_I16_U16                     = 42,
	EXP_DICT_I16_U08                     = 43,
	EXP_DICT_DBL_U32                     = 44,
	EXP_DICT_DBL_U16                     = 45,
	EXP_DICT_DBL_U08                     = 46,
	EXP_DICT_STR_U32                     = 47,
	EXP_DICT_STR_U16                     = 48,
	EXP_DICT_STR_U08                     = 49,
	EXP_ALP_DBL                          = 50,
	EXP_ALP_RD_DBL                       = 51,
	EXP_FSST                             = 52,
	EXP_FSST12                           = 53,
	EXP_FSST_DELTA                       = 54,
	EXP_FSST12_DELTA                     = 55,
	EXP_RLE_I64_U16                      = 56,
	EXP_RLE_I32_U16                      = 57,
	EXP_RLE_I16_U16                      = 58,
	EXP_RLE_DBL_U16                      = 59,
	EXP_RLE_STR_U16                      = 60,
	EXP_DELTA_I64                        = 61,
	EXP_DELTA_I32                        = 62,
	EXP_DELTA_I16                        = 63,
	EXP_FFOR_SLPATCH_I64                 = 64,
	EXP_FFOR_SLPATCH_I32                 = 65,
	EXP_FFOR_SLPATCH_I16                 = 66,
	EXP_DICT_I64_FFOR_SLPATCH_U32        = 67,
	EXP_DICT_I64_FFOR_SLPATCH_U16        = 68,
	EXP_DICT_I64_FFOR_SLPATCH_U08        = 69,
	EXP_DICT_I32_FFOR_SLPATCH_U32        = 70,
	EXP_DICT_I32_FFOR_SLPATCH_U16        = 71,
	EXP_DICT_I32_FFOR_SLPATCH_U08        = 72,
	EXP_DICT_I16_FFOR_SLPATCH_U16        = 73,
	EXP_DICT_I16_FFOR_SLPATCH_U08        = 74,
	EXP_DICT_DBL_FFOR_SLPATCH_U32        = 75,
	EXP_DICT_DBL_FFOR_SLPATCH_U16        = 76,
	EXP_DICT_DBL_FFOR_SLPATCH_U08        = 77,
	EXP_DICT_STR_FFOR_SLPATCH_U32        = 78,
	EXP_DICT_STR_FFOR_SLPATCH_U16        = 79,
	EXP_DICT_STR_FFOR_SLPATCH_U08        = 80,
	EXP_FSST_DICT_STR_FFOR_SLPATCH_U32   = 81,
	EXP_FSST_DICT_STR_FFOR_SLPATCH_U16   = 82,
	EXP_FSST_DICT_STR_FFOR_SLPATCH_U08   = 83,
	EXP_NULL_DBL                         = 84,
	EXP_NULL_I16                         = 85,
	EXP_RLE_I64_SLPATCH_U16              = 86,
	EXP_RLE_I32_SLPATCH_U16              = 87,
	EXP_RLE_I16_SLPATCH_U16              = 88,
	EXP_RLE_DBL_SLPATCH_U16              = 89,
	EXP_RLE_STR_SLPATCH_U16              = 90,
	EXP_FSST_DICT_STR_U32                = 91,
	EXP_FSST_DICT_STR_U16                = 92,
	EXP_FSST_DICT_STR_U08                = 93,
	EXP_FSST_DELTA_SLPATCH               = 94,
	EXP_FSST12_DELTA_SLPATCH             = 95,
	EXP_FSST12_DICT_STR_U32              = 96,
	EXP_FSST12_DICT_STR_U16              = 97,
	EXP_FSST12_DICT_STR_U08              = 98,
	EXP_FSST12_DICT_STR_FFOR_SLPATCH_U32 = 99,
	EXP_FSST12_DICT_STR_FFOR_SLPATCH_U16 = 100,
	EXP_FSST12_DICT_STR_FFOR_SLPATCH_U08 = 101,
	EXP_FSST12_DICT_STR_FFOR_U32         = 102,
	EXP_FSST12_DICT_STR_FFOR_U16         = 103,
	EXP_FSST12_DICT_STR_FFOR_U08         = 104,
	EXP_UNCOMPRESSED_I08                 = 105,
	EXP_CONSTANT_I08                     = 106,
	EXP_DICT_I08_U08                     = 107,
	EXP_DICT_I08_FFOR_U08                = 108,
	EXP_FFOR_I08                         = 109,
	WIZARD_CHOOSE_DICT                   = 110,
	EXP_RLE_I08_U16                      = 111,
	EXP_DELTA_I08                        = 112,
	EXP_FFOR_SLPATCH_I08                 = 113,
	EXP_RLE_I08_SLPATCH_U16              = 114,
	EXP_DICT_I08_FFOR_SLPATCH_U08        = 115,
	EXP_FREQUENCY_DBL                    = 116,
	EXP_FREQUENCY_I08                    = 117,
	EXP_FREQUENCY_I16                    = 118,
	EXP_FREQUENCY_I32                    = 119,
	EXP_FREQUENCY_I64                    = 120,
	EXP_FREQUENCY_STR                    = 121,
	EXP_CROSS_RLE_I08                    = 122,
	EXP_CROSS_RLE_I16                    = 123,
	EXP_CROSS_RLE_I32                    = 124,
	EXP_CROSS_RLE_I64                    = 125,
	EXP_CROSS_RLE_DBL                    = 126,
	EXP_CROSS_RLE_STR                    = 127,
	EXP_NULL_I32                         = 128,
	EXP_CONSTANT_FLT                     = 129,
	EXP_UNCOMPRESSED_FLT                 = 130,
	EXP_NULL_FLT                         = 131,
	EXP_ALP_FLT                          = 132,
	EXP_RLE_FLT_U16                      = 133,
	EXP_DICT_FLT_FFOR_U16                = 134,
	EXP_RLE_FLT_SLPATCH_U16              = 135,
	EXP_ALP_RD_FLT                       = 136,
	EXP_FREQUENCY_FLT                    = 137,
	EXP_CROSS_RLE_FLT                    = 138,
	EXP_DICT_FLT_FFOR_U08                = 139,
	EXP_DICT_FLT_FFOR_SLPATCH_U08        = 140,
	EXP_DICT_FLT_FFOR_SLPATCH_U16        = 141,
	EXP_DICT_FLT_FFOR_U32                = 142,
	EXP_DICT_FLT_FFOR_SLPATCH_U32        = 143
};

inline const OperatorToken (&EnumValuesOperatorToken())[144] {
	static const OperatorToken values[] = {OperatorToken::INVALID,
	                                       OperatorToken::EXP_EQUAL,
	                                       OperatorToken::EXP_CONSTANT_I64,
	                                       OperatorToken::EXP_CONSTANT_I32,
	                                       OperatorToken::EXP_CONSTANT_I16,
	                                       OperatorToken::EXP_CONSTANT_DBL,
	                                       OperatorToken::EXP_CONSTANT_STR,
	                                       OperatorToken::EXP_CONSTANT_U08,
	                                       OperatorToken::EXP_STRUCT,
	                                       OperatorToken::EXP_UNCOMPRESSED_STR,
	                                       OperatorToken::EXP_UNCOMPRESSED_I64,
	                                       OperatorToken::EXP_UNCOMPRESSED_I16,
	                                       OperatorToken::EXP_UNCOMPRESSED_I32,
	                                       OperatorToken::EXP_UNCOMPRESSED_DBL,
	                                       OperatorToken::EXP_UNCOMPRESSED_U08,
	                                       OperatorToken::EXP_FFOR_I64,
	                                       OperatorToken::EXP_FFOR_I32,
	                                       OperatorToken::EXP_FFOR_I16,
	                                       OperatorToken::EXP_DICT_I64_FFOR_U32,
	                                       OperatorToken::EXP_DICT_I64_FFOR_U16,
	                                       OperatorToken::EXP_DICT_I64_FFOR_U08,
	                                       OperatorToken::EXP_DICT_I32_FFOR_U32,
	                                       OperatorToken::EXP_DICT_I32_FFOR_U16,
	                                       OperatorToken::EXP_DICT_I32_FFOR_U08,
	                                       OperatorToken::EXP_DICT_I16_FFOR_U16,
	                                       OperatorToken::EXP_DICT_I16_FFOR_U08,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_U32,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_U16,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_U08,
	                                       OperatorToken::EXP_DICT_STR_FFOR_U32,
	                                       OperatorToken::EXP_DICT_STR_FFOR_U16,
	                                       OperatorToken::EXP_DICT_STR_FFOR_U08,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_U32,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_U16,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_U08,
	                                       OperatorToken::WIZARD_DICTIONARY_ENCODE,
	                                       OperatorToken::EXP_DICT_I64_U32,
	                                       OperatorToken::EXP_DICT_I64_U16,
	                                       OperatorToken::EXP_DICT_I64_U08,
	                                       OperatorToken::EXP_DICT_I32_U32,
	                                       OperatorToken::EXP_DICT_I32_U16,
	                                       OperatorToken::EXP_DICT_I32_U08,
	                                       OperatorToken::EXP_DICT_I16_U16,
	                                       OperatorToken::EXP_DICT_I16_U08,
	                                       OperatorToken::EXP_DICT_DBL_U32,
	                                       OperatorToken::EXP_DICT_DBL_U16,
	                                       OperatorToken::EXP_DICT_DBL_U08,
	                                       OperatorToken::EXP_DICT_STR_U32,
	                                       OperatorToken::EXP_DICT_STR_U16,
	                                       OperatorToken::EXP_DICT_STR_U08,
	                                       OperatorToken::EXP_ALP_DBL,
	                                       OperatorToken::EXP_ALP_RD_DBL,
	                                       OperatorToken::EXP_FSST,
	                                       OperatorToken::EXP_FSST12,
	                                       OperatorToken::EXP_FSST_DELTA,
	                                       OperatorToken::EXP_FSST12_DELTA,
	                                       OperatorToken::EXP_RLE_I64_U16,
	                                       OperatorToken::EXP_RLE_I32_U16,
	                                       OperatorToken::EXP_RLE_I16_U16,
	                                       OperatorToken::EXP_RLE_DBL_U16,
	                                       OperatorToken::EXP_RLE_STR_U16,
	                                       OperatorToken::EXP_DELTA_I64,
	                                       OperatorToken::EXP_DELTA_I32,
	                                       OperatorToken::EXP_DELTA_I16,
	                                       OperatorToken::EXP_FFOR_SLPATCH_I64,
	                                       OperatorToken::EXP_FFOR_SLPATCH_I32,
	                                       OperatorToken::EXP_FFOR_SLPATCH_I16,
	                                       OperatorToken::EXP_DICT_I64_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_DICT_I64_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_I64_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_DICT_I32_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_DICT_I32_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_I32_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_DICT_I16_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_I16_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_DBL_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_DICT_STR_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_DICT_STR_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_STR_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_FSST_DICT_STR_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_NULL_DBL,
	                                       OperatorToken::EXP_NULL_I16,
	                                       OperatorToken::EXP_RLE_I64_SLPATCH_U16,
	                                       OperatorToken::EXP_RLE_I32_SLPATCH_U16,
	                                       OperatorToken::EXP_RLE_I16_SLPATCH_U16,
	                                       OperatorToken::EXP_RLE_DBL_SLPATCH_U16,
	                                       OperatorToken::EXP_RLE_STR_SLPATCH_U16,
	                                       OperatorToken::EXP_FSST_DICT_STR_U32,
	                                       OperatorToken::EXP_FSST_DICT_STR_U16,
	                                       OperatorToken::EXP_FSST_DICT_STR_U08,
	                                       OperatorToken::EXP_FSST_DELTA_SLPATCH,
	                                       OperatorToken::EXP_FSST12_DELTA_SLPATCH,
	                                       OperatorToken::EXP_FSST12_DICT_STR_U32,
	                                       OperatorToken::EXP_FSST12_DICT_STR_U16,
	                                       OperatorToken::EXP_FSST12_DICT_STR_U08,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_SLPATCH_U32,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_U32,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_U16,
	                                       OperatorToken::EXP_FSST12_DICT_STR_FFOR_U08,
	                                       OperatorToken::EXP_UNCOMPRESSED_I08,
	                                       OperatorToken::EXP_CONSTANT_I08,
	                                       OperatorToken::EXP_DICT_I08_U08,
	                                       OperatorToken::EXP_DICT_I08_FFOR_U08,
	                                       OperatorToken::EXP_FFOR_I08,
	                                       OperatorToken::WIZARD_CHOOSE_DICT,
	                                       OperatorToken::EXP_RLE_I08_U16,
	                                       OperatorToken::EXP_DELTA_I08,
	                                       OperatorToken::EXP_FFOR_SLPATCH_I08,
	                                       OperatorToken::EXP_RLE_I08_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_I08_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_FREQUENCY_DBL,
	                                       OperatorToken::EXP_FREQUENCY_I08,
	                                       OperatorToken::EXP_FREQUENCY_I16,
	                                       OperatorToken::EXP_FREQUENCY_I32,
	                                       OperatorToken::EXP_FREQUENCY_I64,
	                                       OperatorToken::EXP_FREQUENCY_STR,
	                                       OperatorToken::EXP_CROSS_RLE_I08,
	                                       OperatorToken::EXP_CROSS_RLE_I16,
	                                       OperatorToken::EXP_CROSS_RLE_I32,
	                                       OperatorToken::EXP_CROSS_RLE_I64,
	                                       OperatorToken::EXP_CROSS_RLE_DBL,
	                                       OperatorToken::EXP_CROSS_RLE_STR,
	                                       OperatorToken::EXP_NULL_I32,
	                                       OperatorToken::EXP_CONSTANT_FLT,
	                                       OperatorToken::EXP_UNCOMPRESSED_FLT,
	                                       OperatorToken::EXP_NULL_FLT,
	                                       OperatorToken::EXP_ALP_FLT,
	                                       OperatorToken::EXP_RLE_FLT_U16,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_U16,
	                                       OperatorToken::EXP_RLE_FLT_SLPATCH_U16,
	                                       OperatorToken::EXP_ALP_RD_FLT,
	                                       OperatorToken::EXP_FREQUENCY_FLT,
	                                       OperatorToken::EXP_CROSS_RLE_FLT,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_U08,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_SLPATCH_U08,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_SLPATCH_U16,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_U32,
	                                       OperatorToken::EXP_DICT_FLT_FFOR_SLPATCH_U32};
	return values;
}

inline const char* const* EnumNamesOperatorToken() {
	static const char* const names[145] = {"INVALID",
	                                       "EXP_EQUAL",
	                                       "EXP_CONSTANT_I64",
	                                       "EXP_CONSTANT_I32",
	                                       "EXP_CONSTANT_I16",
	                                       "EXP_CONSTANT_DBL",
	                                       "EXP_CONSTANT_STR",
	                                       "EXP_CONSTANT_U08",
	                                       "EXP_STRUCT",
	                                       "EXP_UNCOMPRESSED_STR",
	                                       "EXP_UNCOMPRESSED_I64",
	                                       "EXP_UNCOMPRESSED_I16",
	                                       "EXP_UNCOMPRESSED_I32",
	                                       "EXP_UNCOMPRESSED_DBL",
	                                       "EXP_UNCOMPRESSED_U08",
	                                       "EXP_FFOR_I64",
	                                       "EXP_FFOR_I32",
	                                       "EXP_FFOR_I16",
	                                       "EXP_DICT_I64_FFOR_U32",
	                                       "EXP_DICT_I64_FFOR_U16",
	                                       "EXP_DICT_I64_FFOR_U08",
	                                       "EXP_DICT_I32_FFOR_U32",
	                                       "EXP_DICT_I32_FFOR_U16",
	                                       "EXP_DICT_I32_FFOR_U08",
	                                       "EXP_DICT_I16_FFOR_U16",
	                                       "EXP_DICT_I16_FFOR_U08",
	                                       "EXP_DICT_DBL_FFOR_U32",
	                                       "EXP_DICT_DBL_FFOR_U16",
	                                       "EXP_DICT_DBL_FFOR_U08",
	                                       "EXP_DICT_STR_FFOR_U32",
	                                       "EXP_DICT_STR_FFOR_U16",
	                                       "EXP_DICT_STR_FFOR_U08",
	                                       "EXP_FSST_DICT_STR_FFOR_U32",
	                                       "EXP_FSST_DICT_STR_FFOR_U16",
	                                       "EXP_FSST_DICT_STR_FFOR_U08",
	                                       "WIZARD_DICTIONARY_ENCODE",
	                                       "EXP_DICT_I64_U32",
	                                       "EXP_DICT_I64_U16",
	                                       "EXP_DICT_I64_U08",
	                                       "EXP_DICT_I32_U32",
	                                       "EXP_DICT_I32_U16",
	                                       "EXP_DICT_I32_U08",
	                                       "EXP_DICT_I16_U16",
	                                       "EXP_DICT_I16_U08",
	                                       "EXP_DICT_DBL_U32",
	                                       "EXP_DICT_DBL_U16",
	                                       "EXP_DICT_DBL_U08",
	                                       "EXP_DICT_STR_U32",
	                                       "EXP_DICT_STR_U16",
	                                       "EXP_DICT_STR_U08",
	                                       "EXP_ALP_DBL",
	                                       "EXP_ALP_RD_DBL",
	                                       "EXP_FSST",
	                                       "EXP_FSST12",
	                                       "EXP_FSST_DELTA",
	                                       "EXP_FSST12_DELTA",
	                                       "EXP_RLE_I64_U16",
	                                       "EXP_RLE_I32_U16",
	                                       "EXP_RLE_I16_U16",
	                                       "EXP_RLE_DBL_U16",
	                                       "EXP_RLE_STR_U16",
	                                       "EXP_DELTA_I64",
	                                       "EXP_DELTA_I32",
	                                       "EXP_DELTA_I16",
	                                       "EXP_FFOR_SLPATCH_I64",
	                                       "EXP_FFOR_SLPATCH_I32",
	                                       "EXP_FFOR_SLPATCH_I16",
	                                       "EXP_DICT_I64_FFOR_SLPATCH_U32",
	                                       "EXP_DICT_I64_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_I64_FFOR_SLPATCH_U08",
	                                       "EXP_DICT_I32_FFOR_SLPATCH_U32",
	                                       "EXP_DICT_I32_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_I32_FFOR_SLPATCH_U08",
	                                       "EXP_DICT_I16_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_I16_FFOR_SLPATCH_U08",
	                                       "EXP_DICT_DBL_FFOR_SLPATCH_U32",
	                                       "EXP_DICT_DBL_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_DBL_FFOR_SLPATCH_U08",
	                                       "EXP_DICT_STR_FFOR_SLPATCH_U32",
	                                       "EXP_DICT_STR_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_STR_FFOR_SLPATCH_U08",
	                                       "EXP_FSST_DICT_STR_FFOR_SLPATCH_U32",
	                                       "EXP_FSST_DICT_STR_FFOR_SLPATCH_U16",
	                                       "EXP_FSST_DICT_STR_FFOR_SLPATCH_U08",
	                                       "EXP_NULL_DBL",
	                                       "EXP_NULL_I16",
	                                       "EXP_RLE_I64_SLPATCH_U16",
	                                       "EXP_RLE_I32_SLPATCH_U16",
	                                       "EXP_RLE_I16_SLPATCH_U16",
	                                       "EXP_RLE_DBL_SLPATCH_U16",
	                                       "EXP_RLE_STR_SLPATCH_U16",
	                                       "EXP_FSST_DICT_STR_U32",
	                                       "EXP_FSST_DICT_STR_U16",
	                                       "EXP_FSST_DICT_STR_U08",
	                                       "EXP_FSST_DELTA_SLPATCH",
	                                       "EXP_FSST12_DELTA_SLPATCH",
	                                       "EXP_FSST12_DICT_STR_U32",
	                                       "EXP_FSST12_DICT_STR_U16",
	                                       "EXP_FSST12_DICT_STR_U08",
	                                       "EXP_FSST12_DICT_STR_FFOR_SLPATCH_U32",
	                                       "EXP_FSST12_DICT_STR_FFOR_SLPATCH_U16",
	                                       "EXP_FSST12_DICT_STR_FFOR_SLPATCH_U08",
	                                       "EXP_FSST12_DICT_STR_FFOR_U32",
	                                       "EXP_FSST12_DICT_STR_FFOR_U16",
	                                       "EXP_FSST12_DICT_STR_FFOR_U08",
	                                       "EXP_UNCOMPRESSED_I08",
	                                       "EXP_CONSTANT_I08",
	                                       "EXP_DICT_I08_U08",
	                                       "EXP_DICT_I08_FFOR_U08",
	                                       "EXP_FFOR_I08",
	                                       "WIZARD_CHOOSE_DICT",
	                                       "EXP_RLE_I08_U16",
	                                       "EXP_DELTA_I08",
	                                       "EXP_FFOR_SLPATCH_I08",
	                                       "EXP_RLE_I08_SLPATCH_U16",
	                                       "EXP_DICT_I08_FFOR_SLPATCH_U08",
	                                       "EXP_FREQUENCY_DBL",
	                                       "EXP_FREQUENCY_I08",
	                                       "EXP_FREQUENCY_I16",
	                                       "EXP_FREQUENCY_I32",
	                                       "EXP_FREQUENCY_I64",
	                                       "EXP_FREQUENCY_STR",
	                                       "EXP_CROSS_RLE_I08",
	                                       "EXP_CROSS_RLE_I16",
	                                       "EXP_CROSS_RLE_I32",
	                                       "EXP_CROSS_RLE_I64",
	                                       "EXP_CROSS_RLE_DBL",
	                                       "EXP_CROSS_RLE_STR",
	                                       "EXP_NULL_I32",
	                                       "EXP_CONSTANT_FLT",
	                                       "EXP_UNCOMPRESSED_FLT",
	                                       "EXP_NULL_FLT",
	                                       "EXP_ALP_FLT",
	                                       "EXP_RLE_FLT_U16",
	                                       "EXP_DICT_FLT_FFOR_U16",
	                                       "EXP_RLE_FLT_SLPATCH_U16",
	                                       "EXP_ALP_RD_FLT",
	                                       "EXP_FREQUENCY_FLT",
	                                       "EXP_CROSS_RLE_FLT",
	                                       "EXP_DICT_FLT_FFOR_U08",
	                                       "EXP_DICT_FLT_FFOR_SLPATCH_U08",
	                                       "EXP_DICT_FLT_FFOR_SLPATCH_U16",
	                                       "EXP_DICT_FLT_FFOR_U32",
	                                       "EXP_DICT_FLT_FFOR_SLPATCH_U32",
	                                       nullptr};
	return names;
}

inline const char* EnumNameOperatorToken(OperatorToken e) {
	if (::flatbuffers::IsOutRange(e, OperatorToken::INVALID, OperatorToken::EXP_DICT_FLT_FFOR_SLPATCH_U32))
		return "";
	const size_t index = static_cast<size_t>(e);
	return EnumNamesOperatorToken()[index];
}

} // namespace fastlanes

#endif // FLATBUFFERS_GENERATED_OPERATORTOKEN_FASTLANES_H_
