#ifndef FLS_FILTER_FILTER_HPP
#define FLS_FILTER_FILTER_HPP

namespace fastlanes {

// chunk
// scanner(chunk)
// physical_exp = interpret(rpn, chunk)
// for loop {
// next
// execute
// }
}
#endif // FLS_FILTER_FILTER_HPP
